/*  libim - Library of Instant Messaging and Presence enabler
 * im.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */

/**
 *@mainpage LiPS Instant Message Enabler APIs
 *@version 1.0
 *@author ZHAO Liangjing liangjing.zhao@orange-ftgroup.com
 *
 *@section abstract Abstract
 *
 *The purpose of LiPS Instance Message (IM) Enabler is to define a set of
 *protocol agnostic APIs to facilitate IM application development for
 *LiPS compatible platform. The standard protocols include: IETF XMPP, IETF
 *SIP/SIMPLE, OMA Wireless Village (IMPS).
 *
 *In generally, IM functionalities are mainly classified as 4 categories:
 * - Message
 * - Presence
 * - Group
 * - Content sharing
 *
 *As the first step and the mobile usage scenario, we are going to standardize
 *the APIs for Message, Presence and a part of Group functionalities. Some advanced
 *features are absent in this version, such as privacy control, group chatting, message
 *session etc. Those absent features will be added in the future.
 *
 *For more detail, please read related pages:
 *  - @ref concept
 *  - @ref model
 *
 *@page concept Concepts
 *Before to use the IM enabler APIs, the programmer should understand some
 *some basic concepts explained in following sections.
 *
 *@section IM_URI IM URI Address
 *
 *IM URI address is used to identify an user of the IM Service.
 *
 *For the public IM protocols standardized by IETF/OMA, such as IMPS, SIP/SIMPLE
 *or IMPS, the IM URI are defined in the similar formats:
 *
 *             <code>scheme:user\@domain</code>
 *
 *Unusually, XMPP, SIP/SIMPLE or IMPS address URI is composed of several parts:
 *   - Scheme;
 *   - User;
 *   - Domain;
 *   - Resource;
 *
 *For example:
 *   - An Xmpp uri <code>jabber:john.smith\@jabber.org/mobile</code>:
 *     - scheme: jabber;
 *     - user: john.smith;
 *     - domain: jabber.org;
 *     - resource: mobile;
 *
 *   - An SIP uri <code>sip:john.smith\@somedomain.com</code>
 *     - scheme: sip;
 *     - user: john.smith
 *     - domain: somedomain.com
 *
 *   - An IMPS uri <code>wv:john.smith/friends\@imps.com</code>
 *     - scheme: wv;
 *     - user: john.smith
 *     - domain: imps.com
 *     - resource: friends;
 *
 *Besides these parts introduced above, protocols may also support some extension
 *values. A good example is a SIP uri could be
 *\par
 *      <code>sip:+86-10-87654321\@somedomain.com;user=phone</code>
 *
 *Here the sip uri has an extension variable named \c 'user', and the value is \c 'phone'.
 *
 *In LiPS IM enabler, the #im_uri_t structure is defined to represent an URI.
 *
 *@section IM_Msg Message
 *
 *Messaging is a means for sending small, simple messages that are
 *delivered immediately to online users. Message functionalities of
 *popular IM applications in PC provide: 
 *   - Send / receive chatting message
 *   - Send / receive group chatting message
 *   - Receive broadcast message
 *   - Chatting session
 *   .
 *IN this LiPS IM enabler, we'll going to support the first three
 *functions above. But how to identify a group chatting destination
 *is still an open issue.
 *
 *The #im_msg_t structure is defined to represent a piece of IM message.
 *
 *@section IM_Presence Presence
 *IM Presence is an ability to tell the status or availabilities of
 *an IM user, such as: the user is online and available to chat; the
 *user is busy now; the user is connected by a mobile phone etc.
 *
 *As the convergence of IM application and embedded devices, more and
 *more status information could be publish as IM user's PRESENTITY, 
 *for instance, user's geography information and user's device 
 *availability / capacities.
 *
 *IM Presence functionalities include:
 *   -	User presence publish
 *   -	Receive buddy presence notification & pull buddy's presence
 *   -	Presence notification subscription & authorization
 *   -	Privacy control
 *   .
 *IM enabler supports the first three functions currently, while
 *the 4th point will be supported in future.
 *
 *The #im_presence_t structure is defined to encapsulate presence
 *information of a user.
 *
 *For privacy reason, IM user can receive other's presence only after
 *he / her got the permission from other user. This is under the 
 *control of presence subscription mechanism. For example: User A
 *would like to receive user B's presence change notification, then
 *user A should send out a presence subscription firstly to user B.
 *When receiving the subscription, user B can choose to accept or 
 *refuse by sending back an acceptance message. If user B accept user
 *A's subscription, when A is online and B changes his presence, user
 *A will be notified. Once A got the permission from B, B can also recuse
 *the permission at anytime. If B recuse the permission, user A won't
 *received B's notification anymore.
 *
 *@page model Model
 *@section IM_Event	Event
 *LiPS IM enabler keeps consistency with LiPS programming model. 
 *All the Messaging and Presence functionalities are achieved by
 *Asynchronous Requests & Events. Events are classified as two types:
 *asynchronous events and unsolicited events.
 *
 *@subsection async_evt	Asynchronous Event
 *Asynchronous event is the response to the asynchronous request from
 *application. 
 *
 *IM enabler provides a set of APIs to allow application to send asynchronous
 *request.  Each invocation of these APIs will pass a callback function to IM
 *enabler, and return a session unique Request ID to identify the request
 *invocation. The callback function will be invoked by IM enabler when the
 *response event happens and the event is identified by the Request ID.
 *
 *For instance: the IM application sends out a text message, an asynchronous
 *event will pop up to tell application if the message have been sent out
 *successfully or not.
 *
 *@subsection unsoli_evt	Unsolicited Event
 *Unsolicited event is initiated by remote user, network server or other IM
 *enabler users. 
 *
 *IM enabler provides a set of APIs to allow application registering callback
 *functions to listen to the unsolicited events. When an unsolicited event is
 *coming, the related callback function will be invoked. 
 *
 *For instance: the IM application registered to listening incoming message event.
 *When a message from a buddy is arriving, an unsolicited event will pop up so
 *that the application can catch the information about the message.
 *
 *@subsection evt_types Event types
 *Respected to the functionalities, IM enabler defined the following event types:
 *<TABLE>
 *  <TR><TD>Func</TD><TD>Async requests</TD><TD>Unsolicited events</TD></TR>
 *  <TR><TD>Message</TD><TD>Send out a message</TD><TD>A message received</TD></TR>
 *  <TR><TD>Presence</TD><TD>Subscribe to a user<BR>
 *                           Authorize a subscription<BR>
 *                           Publish presence change<BR>
 *                           Presence probe</TD>
 *                       <TD>Presence notification<BR>
 *                           Subscription received<BR>
 *                           Authorization received</TD></TR>
 *  <TR><TD>Other</TD><TD>Request Login<BR>
 *                        Request Logout<BR></TD>
 *                    <TD>Other session login<BR>
 *                        Other session logout<BR>
 *                        Error received</TD></TR>
 *</TABLE>
 *
 *@section session	Session
 *Session is the way that LiPS IM enabler keeps execution context of application.
 *All the requests and events should happen in a certain execution context. That
 *means all the event related APIs should be invocated within a session. Before
 *using any event related APIs, applications have to open a session, and if the
 *application won't use any event API, application should close the session. 
 *
 *With session mechanism, LiPS IM enabler can keep the state of the IM enabler
 *user (the applications), and share the server connection with different session.
 *Thus, an application can open several sessions in the same time, each session 
 *form a different IM server connection (for multi account supporting); while an
 *IM server connection can also serve several IM applications in the same time,
 *the applications can be recognized by their sessions.
 *
 *The type #im_sid_t is defined the identifier of a session.
 */

#ifndef __LIPS_IM_H
#define __LIPS_IM_H

#include "imtype.h"
#include "imerror.h"
#include "imcontact.h"
#include "imevent.h"
#include "immsg.h"
#include "impresence.h"
#include "imuri.h"

#define IM_VERSION_MAJOR 0
#define IM_VERSION_MINOR 1

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  /**
   *@defgroup LiPS_IM_API LiPS_IM_API
   *@{
   */

  /**
   *@brief get version information
   *
   *@param[in]  major major number of LiPS IM Enabler API.
   *@param[in]  minor minor number of LiPS IM Enabler API.
   */
  void im_version (int *major, int *minor);

  /**
   *@brief initialize necessary resources for im enabler.
   *
   *@return an #im_err_t value:
   *           - #IM_ERROR_NONE: no error.
   *           - others: refer to #im_err_t.
   */
  im_err_t im_init (void);

  /**
   *@brief release resources used by im enabler.
   *
   *@return an #im_err_t value:
   *           - #IM_ERROR_NONE: no error.
   *           - others: refer to #im_err_t.
   */
  im_err_t im_finalize (void);

  /**
   *@brief open an im session.
   *
   *@param[out] pSid return the new sid. It will be 0 if any error happens.
   *@param[in]  protocol specify the protocol of wanted session.
   *@param[in]  version specify the version of wanted session.
   *@param[in]  server server url.
   *@param[in]  port port of server.
   *@param[in]  user login name.
   *@return an #im_err_t value:
   *           - #IM_ERROR_NONE: no error.
   *           - others: refer to #im_err_t.
   */
  im_err_t im_open_session (im_sid_t *pSid,
                            im_protocol_t protocol,
                            char *version,
                            char *server, char *port, char *user);

  /**
   *@brief close an im session.
   *
   *@param[in]  sid the id of the session which will be closed.
   *@return an #im_err_t value.
   *           - #IM_ERROR_NONE: no error.
   *           - others: refer to #im_err_t.
   */
  im_err_t im_close_session (im_sid_t sid);

  /* 
   *@brief start im enabler event service.
   *
   *@return an #im_err_t value.
   */
//   im_err_t im_start();

  /* 
   *@brief stop im enabler event service.
   *
   *@return an #im_err_t value.
   */
//   im_err_t im_stop();

  /**
   *@} //end of LiPS_IM_API group
   */

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif /* __LIPS_IM_H*/

