/*  libim - Library of Instant Messaging and Presence enabler
 * imcontact.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file imcontact.h
 *API definition of the Instance Message Interface - IM Contact
 *
 *@version 0.1
 */

#ifndef __LIPS_IM_CONTACT_H
#define __LIPS_IM_CONTACT_H

#include "imerror.h"
#include "imtype.h"
#include "imuri.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  /**
   *@ingroup LiPS_IM_API
   *@defgroup im_roster IM_ROSTER
   *@{
   */

  /**
  *@brief Subscribtion type enum.
  *
  *Each contactor in the buddy list has some relationship with buddy list owner.
   *Suppose: A is the owner of a buddy list, B is one contactor in the buddy list:
   */
  typedef enum
  {
    IM_SUB_INVALID,             ///< Invalid relationship
    IM_SUB_NONE,                ///< A doesn't subscribe B, B doesn't subscribe A.
    IM_SUB_TO,                  ///< A subscribes B, B doesn't subscribe A.
    IM_SUB_FROM,                ///< A doesn't subscribe B, B subscribes A.
    IM_SUB_BOTH                 ///< A subscribes B, B subscribes A also.
  } im_subs_state_t;

  /**
   *@brief convert an #im_subs_state_t value to a string.
   *
   *@param[in]  state the subscription type value.
   *@return a string value according to state.
   *        This string should not be released by programmers.
   */
  char *im_subs_state_2_str (im_subs_state_t state);

  /**
   *@brief Roster item definition
   *
   *This structure encapsulates all necessary information for an buddy.
   */
  typedef struct
  {
    char *contactor;            ///< Buddy uri string.
    im_subs_state_t subs_state; ///< Subscribe state.
    char *nickname;             ///< Nick name.
    char *group;                ///< Group name.
  }
  im_roster_item_t;

  /**
   *@brief distory an #im_roster_item_t structure
   *
   *@param[in]  item the #im_roster_item_t structure will be released.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   */
  im_err_t im_roster_item_free (im_roster_item_t *item);

  /**
   *@typedef im_roster_t
   *@brief The im_roster_t is an opaque data structure to represent a buddy list.
   *
   *The im_roster_t is a list of #im_roster_item_t values. Programmers can not
   *create an #im_roster_t structure. This structure can only be retrieved from
   *the \c evt_data field of an unsolicited event, such as #IM_EVT_ROSTER event.
   *However, programmers should release this structure after the usage.
   *
   *In the other hand, this list is read only. programmers can not modify
   *this list, such like adding/modifying/removing a buddy (#im_roster_item_t).
   *All of the modifications should be achieved by buddy operations.
   *@see im_buddy_op_t
   *@see im_buddy_op
   */
  typedef struct _im_roster_t im_roster_t;

  /**
   *@brief get the sid of an #im_roster_t.
   *
   *@param[in]  roster the #im_roster_t.
   *@param[out] pSid return the session id of the #im_roster_t.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   */
  im_err_t im_roster_get_sid (im_roster_t *roster, im_sid_t *pSid);

  /**
   *@brief get length of an #im_roster_t.
   *
   *@param[in]  roster the #im_roster_t.
   *@param[out] pLength return the length of the #im_roster_t.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   */
  im_err_t im_roster_get_length (im_roster_t *roster, unsigned int *pLength);

  /**
   *@brief get the #im_roster_item_t in the given position of an #im_roster_t.
   *
   *@param[in]  roster the #im_roster_t.
   *@param[in]  position the position.
   *@param[out] pItem return the item,
   *              NULL if the position is out of the range.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   *        - #IM_ERROR_NULL if the position is out of the range.
   */
  im_err_t im_roster_get_item (im_roster_t *roster, unsigned int position,
                               im_roster_item_t **pItem);

  /**
   *@brief find an #im_roster_item_t in the #im_roster_t by im_uri_t.
   *
   *@param[in]  roster the #im_roster_t.
   *@param[in]  uri the uri.
   *@param[out] pItem return the item,
   *              NULL if no item contains the uri.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   *        - #IM_ERROR_NULL if no item contains the uri.
   */
  im_err_t im_roster_get_by_uri (im_roster_t *roster, char *uri,
                                 im_roster_item_t **pItem);

  /**
   *@brief find the position of an #im_roster_item_t in the given #im_roster_t
   *
   *@param[in]  roster the #im_roster_t to search.
   *@param[in]  item the #im_roster_item_t would be found out.
   *@param[out] pPosition return index of the element containing the item,
   *              or -1 if the data is not found.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   *        - #IM_ERROR_NULL no such item.
   */
  im_err_t im_roster_position (im_roster_t *roster, im_roster_item_t *item,
                               int *pPosition);

  /**
   *@brief destory an #im_roster_t structure
   *
   *@param[in]  roster the #im_roster_t which will be released.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   */
  im_err_t im_roster_free (im_roster_t *roster);

  /**
   *@brief Buddy operation enumeration
   */
  typedef enum
  {
    IM_BUDDY_INFO,              ///< Get Buddy detail information.
    IM_BUDDY_ADD,               ///< Add a Buddy to Roster.
    IM_BUDDY_REMOVE,            ///< Remove a Buddy from Roster.
    IM_BUDDY_UPDATE             ///< Buddy detail Updated.
  } im_buddy_operation_t;

  /**
   *@brief convert an #im_buddy_operation_t value to a string.
   *
   *@param[in]  op_type the #im_buddy_operation_t value.
   *
   *@return a string represent the #im_buddy_operation_t value.
   *        It should not be released by programmers.
   */
  char *im_buddy_operation_2_str (im_buddy_operation_t op_type);

  /**
   *@typedef im_buddy_op_t
   *@brief The im_buddy_op_t is an opaque data structure to represent a buddy operation.
   *
   *The im_buddy_op_t contains an #im_roster_item_t and an #im_buddy_operation_t
   */
  typedef struct _im_buddy_op_t im_buddy_op_t;

  /**
   *@brief create a new #im_buddy_op_t structure
   *
   *@param[in]  buddy an #im_roster_item_t value represents the buddy.
   *             buddy will not be freed when invoke #im_buddy_op_free. In order to
   *             do that, #im_buddy_op_free_all should be invoked.
   *@param[in]  operation an #im_buddy_operation_t value indicates the operation.
   *@param[out] pBuddy_op a pointer to the new #im_buddy_op_t structure.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   *        - #IM_ERROR_NOMEM no enough memory.
   *        - #IM_ERROR_NULL buddy is NULL.
   */
  im_err_t im_buddy_op_new (im_roster_item_t *buddy,
                            im_buddy_operation_t operation,
                            im_buddy_op_t **pBuddy_op);
  /**
   *@brief free the #im_buddy_op_t structure
   *
   *This function will not free the buddy (#im_roster_item_t) structure. Programmers
   *should taket the responsibility to free the buddy.
   *
   *@param[in]  buddy_op the #im_buddy_op_t structure will be released.
   *@return an #im_err_t value.
   */
  im_err_t im_buddy_op_free (im_buddy_op_t *buddy_op);

  /**
   *@brief free the #im_buddy_op_t structure and it's buddy
   *
   *This function will free the buddy (#im_roster_item_t) structure.
   *
   *@param[in]  buddy_op the #im_buddy_op_t structure that will be released.
   *@return an #im_err_t value.
   */
  im_err_t im_buddy_op_free_all (im_buddy_op_t *buddy_op);

  /**
   *@brief get the buddy of an #im_buddy_op_t structure.
   *
   *@param[in]  buddy_op the #im_buddy_op_t structure.
   *@param[out] pBuddy the #im_roster_item_t value represents the buddy.
   *@return an #im_err_t value.
   */
  im_err_t im_buddy_op_get_buddy (im_buddy_op_t *buddy_op,
                                  im_roster_item_t **pBuddy);

  /**
   *@brief get the operation of an #im_buddy_op_t structure.
   *
   *@param[in]  buddy_op the #im_buddy_op_t structure.
   *@param[out] pOperation the operation type.
   *@return an #im_err_t value.
   */
  im_err_t im_buddy_op_get_operation (im_buddy_op_t *buddy_op,
                                      im_buddy_operation_t *pOperation);

  /**
   *@brief set the operation of an #im_buddy_op_t structure.
   *
   *@param[in]  buddy_op the #im_buddy_op_t structure.
   *@param[in]  operation the operation type.
   *@return an #im_err_t value.
   */
  im_err_t im_buddy_op_set_operation (im_buddy_op_t *buddy_op,
                                      im_buddy_operation_t operation);

  /**@} */

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif /* __LIPS_IM_CONTACT_H*/

