/*  libim - Library of Instant Messaging and Presence enabler
 * imevent.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file imevent.h
 *API definition of the Instance Message Interface - Events
 *
 *@version 0.1
 */

#ifndef __LIPS_IM_EVENT_H
#define __LIPS_IM_EVENT_H

#include "imtype.h"
#include "imerror.h"
#include "imuri.h"
#include "impresence.h"
#include "immsg.h"
#include "imcontact.h"
#include <event.h>

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  /**
   *@ingroup LiPS_IM_API
   *@defgroup IM_Event IM_EVENT
   *@{
   */

  /**
   *@brief Type definition for IM request id.
   *
   *Each asynchronous event request will return an id to
   *  identify the request.
   */
  typedef rid_t im_rid_t;

  /**
   *@brief IM Event types enum.
   *
   *Consist to LiPS programming model, IM events are classified by asynchronous
   *   events and unsolicited event.
   *
   *In general, an asynchronous event is caused by applications' requests. ex. application
   *send out a message, an #IM_REQ_MESSAGE will be returned to tell application the message
   *was sent out or not. On the contrary, an unsolicited event is caused by outside of application,
   *such as network or server. ex. a message is received, then an #IM_EVT_MESSAGE will be
   *generated to tell application all the information about this message.
   */
  typedef enum
  {
    IM_EVT_INVALID = -1,

    IM_REQ_LOGIN,               ///<asynchronous event: login request.
    IM_REQ_LOGOUT,              ///<asynchronous event: logout request.
    IM_REQ_ROSTER,              ///<asynchronous event: require buddy list from network.
    IM_REQ_SUBSCRIBE,           ///<asynchronous event: subscribe presence notification request.
    IM_REQ_ACCEPTANCE,          ///<asynchronous event: acceptance reply.
    IM_REQ_PRESENCE,            ///<asynchronous event: self presence change.
    IM_REQ_PRESENCE_PROBE,      ///<asynchronous event: probe an user's presence.
    IM_REQ_MESSAGE,             ///<asynchronous event: send a message.
    IM_REQ_GROUP,               ///<asynchronous event: change group.
    IM_REQ_BUDDY,               ///<asynchronous event: buddy operation request.
    IM_REQ_SYNCH,               ///<asynchronous event: synchronize local buddy modifications with network.

    IM_EVT_ROSTER,              ///<unsolicited event: roster push from server.
    IM_EVT_PRESENCE,            ///<unsolicited event: contactor' presense changed.
    IM_EVT_MESSAGE,             ///<unsolicited event: new messsage arriving.
    IM_EVT_SUBSCRIBE,           ///<unsolicited event: subscribe request arriving.
    IM_EVT_ACCEPTANCE,          ///<unsolicited event: acceptance response arriving.
    IM_EVT_BUDDY,               ///<unsolicited event: buddy event: such as buddy add, buddy remove, buddy update.
    IM_EVT_ERROR,               ///<unsolicited event: any error notification from server.

    IM_EVT_LOGIN,               ///<unsolicited event: this event will be emitted when another session login successfully.
    IM_EVT_LOGOUT               ///<unsolicited event: this event will be emitted when another session logout successfully.
  } im_evt_type_t;

  /**
   *@brief convert an #im_evt_type_t value to a string.
   *
   *@param[in]  etype the event type.
   *@return a string value according to etype.
   *        This string should not be released by programmers.
   */
  char *im_evt_type_2_str (im_evt_type_t etype);

  /**
   *@brief Subscribtion message structure.
   *
   *This structure encapsulates all necessary data for an subscribtion event.
   *@note When sending out a subscribtion request (@ref im_subscribe), the
   *#contactor should be the uri string whom will be subscribed to;
   *while when receiving an #IM_EVT_SUBSCRIBE event, the @c evt_data should be an
   *#im_subs_msg_t structure, and the #contactor should be the uri string who would
   *subscribe to current user.
   */
  typedef struct
  {
    char *contactor;            ///< contactor uri string.
    char *msg;                  ///< Attached Message.
    char *group;                ///< Group name.
    char *nick;                 ///< Nick name.
    bool subscribe;             ///< Subscribe or Unsubscribe
  }
  im_subs_msg_t;

  /**
   *@brief Free an #im_subs_msg_t structure.
   *
   *@param[in]  msg the #im_subs_msg_t will be released.
   *@return an #im_err_t value:
   *        - #IM_ERROR_NONE no error;
   */
  im_err_t im_subs_msg_free (im_subs_msg_t *msg);

  /**
   *@brief Acceptance message structure
   *
   *This structure encapsulates all necessary data for an acceptance event.
   *When user received an subscription from other user, he/her can choose to accept
   *or refuse the subscribtion. This process will be achieved by #IM_REQ_ACCEPTANCE
   *and #IM_EVT_ACCEPTANCE event.
   *
   *application can send out an #IM_REQ_ACCEPTANCE request (@ref im_acceptance) which
   *contains an #im_accept_msg_t structure, in which the #contactor would be the uri
   *string who will be accepted or refused.
   *
   *application can receive an #IM_EVT_ACCEPTANCE event. The @c evt_data should be
   *an #im_accept_msg_t structure, in which the #contactor is the uri string who
   *accepted or refused your previous subscribtion.
   */
  typedef struct
  {
    char *contactor;            ///< contactor uri string.
    bool accept;                ///< Accept or refuse
    char *msg;                  ///< Attached Message.
  }
  im_accept_msg_t;

  /**
   *@brief Free an #im_accept_msg_t structure.
   *
   *@param[in]  msg the #im_accept_msg_t will be released.
   *@return an #im_err_t value:
   *        - #IM_ERROR_NONE no error;
   */
  im_err_t im_accept_msg_free (im_accept_msg_t *msg);

  /**
   *@brief login parameter structure
   *
   *A parameter for the function of login request.
   *Used for place an #IM_REQ_LOGIN request (@ref im_login).
   */
  typedef struct
  {
    im_uri_t *user;             ///< login user
    char *passwd;               ///< password
  }
  im_param_login_t;

  /**
   *@brief destory an #im_param_login_t structure
   *
   *@param[in]  param the pointer to the pointer of an #im_param_login_t which will be destoried.
   *@return an im_err_t value.
   *        - #IM_ERROR_NONE no error.
   */
  im_err_t im_param_login_free (im_param_login_t *param);

  /**
   *@brief Type definition for IM error code
   *
   *IM error code will be only used for asynchronous event.
   *
   *  Not all the requests can be canceled when application try to cancel
   *  an asynchronous event. In this case, IM error code will indicate
   *  what's happed with the cancel request.
   */
  typedef enum
  {
    IM_ERR_CODE_NONE,           ///< No error.
    IM_ERR_CODE_NOSUPPORT,      ///< Not supported.
    IM_ERR_CODE_NETWORK,        ///< Network error.
    IM_ERR_CODE_BAD_URI,        ///< Bad uri.
    IM_ERR_CODE_PASSWORD,       ///< Bad password.
    IM_ERR_CODE_SERVER,         ///< Server close connection.
    IM_ERR_CODE_TRANSACTION,    ///< Transaction error.
    IM_ERR_CODE_MEM,            ///< Not enough memory
    IM_ERR_CODE_OTHER           ///< Other error.
  } im_err_code_t;

  /**
  *@brief structure definition for error message.
  *
  *This structure will be used when an #IM_EVT_ERROR event is emitted.
  *If an event is an #IM_EVT_ERROR, its @c evt_data should be an
  *#im_network_error_t structure.
  */
  typedef struct
  {
    im_err_code_t err_type;     ///< Error type @see im_err_code_t
    char *msg;                  ///< Error message.
  }
  im_network_error_t;

  /**
   *@brief free an #im_network_error_t structure.
   *
   *@param[in]  err the #im_network_error_t will be freed.
   *@return an #im_err_t value:
   *        - #IM_ERROR_NONE no error;
   */
  im_err_t im_network_error_free (im_network_error_t *err);

  /////////////////////////////////////////
  // Unsolicitied event functions
  /////////////////////////////////////////
  /**
   *@brief register an unsolicited event.
   *
   *Before application can receive any unsolicited event, it should
   *register the event type by this function.
   *
   *@param[in]  sid target session id.
   *@param[in]  type the unsolicited event type, refer to #im_evt_type_t.
   *@param[in]  handler the handler for the incoming events.
   *@return an #im_err_t value. refer to #im_err_t.
   */
  im_err_t im_evt_register (sid_t sid,
                            evt_type_t type, evt_handler_t *handler);

  /**
   *@brief unregister an unsolicited event.
   *
   *If application don't want to receive a kind of unsolicited event any
   *more, it should use this function to un-register the event type.
   *
   *@param[in]  sid target session id.
   *@param[in]  type the unsolicited event type, refer to #im_evt_type_t.
   *@return an #im_err_t value. refer to #im_err_t.
   */
  im_err_t im_evt_unregister (sid_t sid, evt_type_t type);

  /////////////////////////////////////////
  // Asynchronous requests functions
  /////////////////////////////////////////

  /**
   *@brief Cancel the asynchronize request event process.
   *
   *This function will cancel the asynchronize request if it is not completed yet.
   *The cancel request maybe take no effect by some reason. Thus, this
   *function will return an error code indicating what's happen to the
   *cancel request.
   *
   *@param[in]  sid session id, see #im_open_session.
   *@param[in]  rid request id to tell which request should be canceled.
   *
   *@return return an #im_err_t value.
   *     - #IM_ERROR_NONE: no error;
   *     - #IM_ERROR_GENERAL: unspecified error;
   *     - EVT_ERROR_WRONG_SID: no such session id;
   *     - EVT_ERROR_WRONG_RID: no such request id;
   *     - EVT_ERROR_EVT_PENDING: event is already pending;
   *     - EVT_ERROR_EVT_COMPLETE: event has been already delivered
   *                               to the client but the client has not released yet;
   *     - #IM_ERROR_NOSUPPORT: cancel operation isn't supported;
   */
  im_err_t im_req_cancel (im_sid_t sid, im_rid_t rid);

  /**
   *@brief Login to im server.
   *
   *This function generates an #IM_REQ_LOGIN event. The login process
   *will take a couple of time. The event will be sent back to the client by
   *calling callback function specified in handler->cb parameter.
   *
   *The evt_data->err_code of #IM_REQ_LOGIN event is contains the error
   *code for login result:
   *     - #IM_ERR_CODE_NONE: login successful;
   *     - #IM_ERR_CODE_NETWORK: login failed, network problem;
   *     - #IM_ERR_CODE_BAD_URI: login failed, bad uri;
   *     - #IM_ERR_CODE_PASSWORD: login failed, password wrong;
   *     - #IM_ERR_CODE_OTHER: login failed, other reason;
   *The evt_data->data should be NULL;
   *
   *@param[in]  sid session id, see #im_open_session.
   *@param[in]  handler a handler structure for delivering the event.
   *@param[in]  pRid return a request id.
   *@param[in]  login an #im_param_login_t pointer indicates login user & password.
   *
   *@return return an #im_err_t value.
   */
  im_err_t im_login (im_sid_t sid,
                     evt_handler_t *handler,
                     im_rid_t *pRid, im_param_login_t *login);

  /**
   *@brief Logout from im server.
   *
   *This function generates an #IM_REQ_LOGOUT event. The logout process
   *will take a couple of time. The event will be sent back to the client by
   *calling callback function specified in handler->cb parameter.
   *
   *Logout request could not be canceled.
   *
   *The evt_data->err_code of #IM_REQ_LOGOUT event is contains the error
   *code for logout result:
   *     - #IM_ERR_CODE_NONE: logout successful;
   *     - #IM_ERR_CODE_NETWORK: logout failed, network problem;
   *     - #IM_ERR_CODE_OTHER: logout failed, other reason;
   *The evt_data->data should be NULL;
   *
   *@param[in]  sid session id, see #im_open_session.
   *@param[in]  handler the callback function for delivering the event.
   *@param[in]  pRid return a request id.
   *
   *@return return an #im_err_t value.
   */
  im_err_t im_logout (im_sid_t sid, evt_handler_t *handler, im_rid_t *pRid);

  /**
   *@brief Send out a message.
   *
   *This function generates an #IM_REQ_MESSAGE event. The event will be
   *sent back to the client by calling callback function specified in handler->cb parameter.
   *
   *@param[in]  sid session id, see #im_open_session.
   *@param[in]  handler a handler structure for delivering the event.
   *@param[in]  pRid return a request id.
   *@param[in]  msg an #im_msg_t pointer contains the message.
   *
   *@return return an #im_err_t value.
   */
  im_err_t im_send_message (im_sid_t sid,
                            evt_handler_t *handler,
                            im_rid_t *pRid, im_msg_t *msg);

  /**
   *@brief Change user's presence.
   *
   *This function generates an #IM_REQ_PRESENCE event. The event will be
   *sent back to the client by calling callback function specified in handler->cb parameter.
   *
   *@param[in]  sid session id, see #im_open_session.
   *@param[in]  handler a handler structure for delivering the event.
   *@param[in]  pRid return a request id.
   *@param[in]  presence an #im_presence_t pointer contains the new presence.
   *
   *@return return an #im_err_t value.
   */
  im_err_t im_change_presence (im_sid_t sid,
                               evt_handler_t *handler,
                               im_rid_t *pRid, im_presence_t *presence);

  /**
   *@brief Probe an user's presence.
   *
   *This function generates an #IM_REQ_PRESENCE_PROBE event. The event will be
   *sent back to the client by calling callback function specified in handler->cb parameter.
   *
   *@param[in]  sid session id, see #im_open_session.
   *@param[in]  handler a handler structure for delivering the event.
   *@param[in]  pRid return a request id.
   *@param[in]  uri the uri string indicate the user.
   *
   *@return return an #im_err_t value.
   */
  im_err_t im_presence_probe (im_sid_t sid,
                              evt_handler_t *handler,
                              im_rid_t *pRid, char *uri);

  /**
   *@brief Subscribe/Unsubscribe to an contactor.
   *
   *This function generates an #IM_REQ_SUBSCRIBE event. The event will be
   *sent back to the client by calling callback function specified in handler->cb parameter.
   *
   *@param[in]  sid session id, see #im_open_session.
   *@param[in]  handler a handler structure for delivering the event.
   *@param[in]  pRid return a request id.
   *@param[in]  subs_msg an #im_subs_msg_t pointer representing the subscription.
   *
   *@return return an #im_err_t value.
   */
  im_err_t im_subscribe (im_sid_t sid,
                         evt_handler_t *handler,
                         im_rid_t *pRid, im_subs_msg_t *subs_msg);

  /**
   *@brief Send out acceptance to a subscriber.
   *
   *This function generates an #IM_REQ_ACCEPTANCE event. The event will be
   *sent back to the client by calling callback function specified in handler->cb parameter.
   *
   *@param[in]  sid session id, see #im_open_session.
   *@param[in]  handler a handler structure for delivering the event.
   *@param[in]  pRid return a request id.
   *@param[in]  accept_msg an #im_accept_msg_t pointer representing the acceptance.
   *
   *@return return an #im_err_t value.
   */
  im_err_t im_acceptance (im_sid_t sid,
                          evt_handler_t *handler,
                          im_rid_t *pRid, im_accept_msg_t *accept_msg);

  /**
   *@brief Send out a rost request to update buddy list.
   *
   *This function generates an #IM_REQ_ROSTER event. The event will be
   *sent back to the client by calling callback function specified in handler->cb parameter.
   *
   *@param[in]  sid session id, see #im_open_session.
   *@param[in]  handler a handler structure for delivering the event.
   *@param[in]  pRid return a request id.
   *
   *@return return an #im_err_t value.
   */
  im_err_t im_get_roster (im_sid_t sid,
                          evt_handler_t *handler, im_rid_t *pRid);

  /**
   *@brief Send out a buddy option request.
   *
   *This function generates an #IM_REQ_BUDDY event. The event will be
   *sent back to the client by calling callback function specified in handler->cb parameter.
   *
   *@param[in]  sid session id, see #im_open_session.
   *@param[in]  handler a handler structure for delivering the event.
   *@param[in]  pRid return a request id.
   *@param[in]  op the buddy operation. refer to #im_buddy_op_t
   *
   *@return return an #im_err_t value.
   */
  im_err_t im_buddy_op (im_sid_t sid,
                        evt_handler_t *handler,
                        im_rid_t *pRid, im_buddy_op_t *op);

  /**
   *@brief Send out a buddy synchronize request.
   *
   *This function generates an #IM_REQ_SYNCH event. The event will be
   *sent back to the client by calling callback function specified in handler->cb parameter.
   *
   *@param[in]  sid session id, see #im_open_session.
   *@param[in]  handler a handler structure for delivering the event.
   *@param[in]  pRid return a request id.
   *
   *@return return an #im_err_t value.
   */
  im_err_t im_buddies_synch (im_sid_t sid,
                             evt_handler_t *handler, im_rid_t *pRid);

  /**
   *@}
   *end group IM_Event
   */

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_IM_EVENT_H */

