/*  libim - Library of Instant Messaging and Presence enabler
 * immsg.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file immsg.h
 *API definition of the Instance Message Interface - IM Message
 *
 *@version 0.1
 */

#ifndef __LIPS_IM_MSG_H
#define __LIPS_IM_MSG_H

#include "imerror.h"
#include "imtype.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  /**
   *@ingroup LiPS_IM_API
   *@defgroup IM_MESSAGE IM_MESSAGE
   *@{
   */

  /**
   *@typedef im_msg_id_t
   *@brief id for an im message.
   */
  typedef uint32_t im_msg_id_t;

  /**
   *@brief Message content type enum.
   *
   *Currently, only Text IM messsage is available.
   */
  typedef enum
  {
    IM_MSG_CTYPE_TEXT           ///< TEXT Message type.
  } im_msg_ctype_t;

  /**
   *@brief Message type enumeration.
   */
  typedef enum
  {
    IM_MSG_TYPE_NORMAL,         ///< Normal Message.
    IM_MSG_TYPE_CHAT,           ///< Chat Message.
    IM_MSG_TYPE_GROUPCHAT,      ///< Group chat Message.
    IM_MSG_TYPE_HEADLINE,       ///< Headline Message.
    IM_MSG_TYPE_ERROR           ///< Message Error.
  } im_msg_type_t;

  /**
   *@brief convert an #im_msg_type_t to a string.
   *
   *@param[in]  msg_type the #im_msg_type_t value should be converted.
   *
   *@return the string. This string should not be freed by user.
   */
  char *im_msg_type_2_str (im_msg_type_t msg_type);

  /**
   *@brief Message state type enum.
   *
   *All the message should belong to one of following state.
   */
  typedef enum
  {
    IM_MSG_STATE_SENDING,       ///< Message is sending out.
    IM_MSG_STATE_SENT,          ///< Message was sent successful.
    IM_MSG_STATE_SEND_FAIL,     ///< Message was sent failed.
    IM_MSG_STATE_RECV,          ///< Received Message.
  } im_msg_state_t;

  /**
   *@struct im_msg_t
   *@brief im_msg_t is an opaque data structure to represent im messages.
   */
  typedef struct _im_msg_t im_msg_t;

  /**
   *@brief create an #im_msg_t structure.
   *
   *@param[in]  sid session id.
   *@param[in]  type message type. @see im_msg_type_t.
   *@param[in]  ctype message content type. @see im_msg_ctype_t.
   *@param[out] msg return the im_msg_t structure.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   *         - #IM_ERROR_NOMEM not enough memory, and \p msg will be \c NULL.
   */
  im_err_t im_msg_new (im_sid_t sid, im_msg_type_t type, im_msg_ctype_t ctype,
                       im_msg_t **msg);

  /**
   *@brief get the sid of a message.
   *
   *@param[in]  msg the message.
   *@param[out] sid return the sid.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   */
  im_err_t im_msg_get_sid (im_msg_t *msg, im_sid_t *sid);

  /**
   *@brief get message id of a message.
   *
   *@param[in]  msg the message.
   *@param[out] msg_id return the message id.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   */
  im_err_t im_msg_get_id (im_msg_t *msg, im_msg_id_t *msg_id);

  /**
   *@brief get the sending/receiving time of a message.
   *
   *@param[in]  msg the message.
   *@param[out] msg_time return the time.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   */
  im_err_t im_msg_get_time (im_msg_t *msg, time_t *msg_time);

  /**
   *@brief get content type of a message.
   *
   *@param[in]  msg the message.
   *@param[out] ctype return the content type.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   */
  im_err_t im_msg_get_ctype (im_msg_t *msg, im_msg_ctype_t *ctype);

  /**
   *@brief get the type of a message.
   *
   *@param[in]  msg the message.
   *@param[out] type return the message type.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   */
  im_err_t im_msg_get_type (im_msg_t *msg, im_msg_type_t *type);

  /**
   *@brief get the state of a message.
   *
   *@param[in]  msg the message.
   *@param[out] msg_state return the message state.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   */
  im_err_t im_msg_get_state (im_msg_t *msg, im_msg_state_t *msg_state);

  /**
   *@brief set the state of a message.
   *
   *@param[in]  msg the message.
   *@param[in]  msg_state the new message state.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   */
  im_err_t im_msg_set_state (im_msg_t *msg, im_msg_state_t msg_state);

  /**
   *@brief get the head length of a message.
   *
   *The message head include recipient list ,from list & encode.
   *
   *@param[in]  msg the message.
   *@param[out] hsize return the message head length.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   */
  im_err_t im_msg_head_length (im_msg_t *msg, size_t *hsize);

  /**
   *@brief get the recipient list of a message.
   *
   *@param[in]  msg the message.
   *@param[out] recipient a pointer to a string return the recipient list.
   *                      The string should be released by programmers after usage.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   *         - #IM_ERROR_NOMEM not enough memory, and \p *recipient will be \c NULL.
   */
  im_err_t im_msg_get_recipient (im_msg_t *msg, char **recipient);

  /**
   *@brief set the recipient list of a message.
   *
   *@param[in]  msg the message.
   *@param[in]  recipient the recipient list.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   *         - #IM_ERROR_NOMEM not enough memory.
   */
  im_err_t im_msg_set_recipient (im_msg_t *msg, char *recipient);

  /**
   *@brief get the 'from' list of a message.
   *
   *from list indicate who sent out the message
   *
   *@param[in]  msg the message.
   *@param[out] from a pointer to a string return the from list.
   *                 The string should be released by programmers after usage.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   *         - #IM_ERROR_NOMEM not enough memory, and \p *from will be \c NULL.
   */
  im_err_t im_msg_get_from (im_msg_t *msg, char **from);

  /**
   *@brief set the from list of a message.
   *
   *from list indicate who sent out the message
   *
   *@param[in]  msg the message.
   *@param[in]  from the from list.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   *         - #IM_ERROR_NOMEM not enough memory.
   */
  im_err_t im_msg_set_from (im_msg_t *msg, char *from);

  /**
   *@brief get the length of a message body.
   *
   *@param[in]  msg the message.
   *@param[out] bsize return the body length, counted by byte.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   */
  im_err_t im_msg_get_body_length (im_msg_t *msg, size_t *bsize);

  /**
   *@brief get the a message body content.
   *
   *@param[in]  msg the message.
   *@param[out] body a pointer to a byte buffer return the body content.
   *                 It should be released by programmers after usage.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   *         - #IM_ERROR_NOMEM not enough memory, and \p *body will be \c NULL.
   */
  im_err_t im_msg_get_body (im_msg_t *msg, void **body);

  /**
   *@brief get the encode of a message body.
   *
   *@param[in]  msg the message.
   *@param[out] encode return the encode. encode is a string such like "en-US", "fr-FR", "zh-CH", "UTF-8"
   *                   This string should be released by programmers after usage.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   *         - #IM_ERROR_NOMEM not enough memory, and \p *encode will be \c NULL.
   */
  im_err_t im_msg_get_encode (im_msg_t *msg, char **encode);

  /**
   *@brief set the a message body.
   *
   *@param[in]  msg the message.
   *@param[in]  body a byte array content the body content.
   *@param[in]  len the length of body in byte.
   *@param[in]  encode if the body is a text string, this parameter indicates the encode of the text.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   *         - #IM_ERROR_NOMEM not enough memory.
   */
  im_err_t im_msg_set_body (im_msg_t *msg, void *body, size_t len,
                            char *encode);

  /**
   *@brief free an #im_msg_t structure.
   *
   *@param[in]  msg the #im_msg_t structure which will be freed.
   *
   *@return an #im_err_t value.
   *         - #IM_ERROR_NONE no error.
   */
  im_err_t im_msg_free (im_msg_t *msg);

  /**
   *@}
   *end group
   */

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif /* __LIPS_IM_MSG_H*/

