/*  libim - Library of Instant Messaging and Presence enabler
 * impresence.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file impresence.h
 *API definition of the Instance Message Interface - IM Presence
 *
 *@version 0.1
 */

#ifndef __LIPS_IM_PRESENCE_H
#define __LIPS_IM_PRESENCE_H

#include "imerror.h"
#include "imtype.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */
  /**
   *@ingroup LiPS_IM_API
   *@defgroup IM_PRESENCE IM_PRESENCE
   *@{
   */

  /**
   *@brief enumeration for presence state.
   */
  typedef enum
  {
    IM_PRESENCE_AVAILABLE,      ///< Buddy is available.
    IM_PRESENCE_AWAY,           ///< Buddy is away.
    IM_PRESENCE_FARAWAY,        ///< Buddy is far away.
    IM_PRESENCE_DND,            ///< Do not disturb the buddy.
    IM_PRESENCE_INVISIBLE,      ///< Invisible.
    IM_PRESENCE_CUSTOM,         ///< State defined by user.
    IM_PRESENCE_OFFLINE         ///< Buddy is offline.
  } im_presence_state_t;

  /**
   *@brief convert a #im_presence_state_t value to a string
   *
   *@param[in]  state the #im_presence_state_t value.
   *
   *@return a string represent the state.
   *        This string should not be released by programmers.
   */
  char *im_presence_2_str (im_presence_state_t state);

  /**
   *@struct im_presence_t
   *@brief im_presence_t is an opaque data structure to represent im presence data.
   */
  typedef struct _im_presence_t im_presence_t;

  /**
   *@brief create a new #im_presence_t structure.
   *
   *@param[in]  prtl the protocol.
   *@param[in]  to a string contain the recipient uri address. For broadcasting user's
   *               presence, this can be set to NULL.
   *@param[out] p a pointer of #im_presence_t pointer to return the new presence structure.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory, and \p *p will be \c NULL;
   */
  im_err_t im_presence_new (im_protocol_t prtl, char *to, im_presence_t **p);

  /**
   *@brief free a #im_presence_t structure.
   *
   *@param[in]  presence the #im_presence_t structure which will be freed.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   */
  im_err_t im_presence_free (im_presence_t *presence);

  /**
   *@brief get the state of a presence structure.
   *
   *@param[in]  presence the #im_presence_t structure.
   *@param[out] state a pointer to a pre-declared #im_presence_state_t
   *                  for returning the state value.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   */
  im_err_t im_presence_get_state (im_presence_t *presence,
                                  im_presence_state_t *state);

  /**
   *@brief set the state of a presence structure.
   *
   *@param[in]  presence the #im_presence_t structure.
   *@param[in]  state the state value.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   */
  im_err_t im_presence_set_state (im_presence_t *presence,
                                  im_presence_state_t state);

  /**
   *@brief get the customized description of a presence structure.
   *
   *@param[in]  presence the #im_presence_t structure.
   *@param[out] desc a pointer to a string to return a copy of the description.
   *                 This string should be released by developers after usage.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no enough memory, and \p *desc will be \c NULL;
   */
  im_err_t im_presence_get_desc (im_presence_t *presence, char **desc);

  /**
   *@brief set the customized state description of a presence structure.
   *
   *@param[in]  presence the presence structure.
   *@param[in]  desc a string of the description.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no enough memory;
   */
  im_err_t im_presence_set_desc (im_presence_t *presence, char *desc);

  /**
   *@brief get the recipient of a presence structure.
   *
   *@param[in]  presence the presence structure.
   *@param[out] to a pointer to a string to return the recipient uri address.
   *               This string should be released by developers after usage.
   *               Normally, for a broadcast presence, this will be \c NULL.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no enough memory, and \p *to will be \c NULL;
   */
  im_err_t im_presence_get_to (im_presence_t *presence, char **to);

  /**
   *@brief set the recipient of a presence structure.
   *
   *@param[in]  presence the presence structure.
   *@param[in]  to a string of the recipient uri address.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no enough memory;
   */
  im_err_t im_presence_set_to (im_presence_t *presence, char *to);

  /**
   *@brief get the sender of a presence structure.
   *
   *@param[in]  presence the presence structure.
   *@param[out] from a pointer to a string to return the sender uri address.
   *             This string should be released by developers after usage.
   *             It will be \c NULL if no sender specified.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no enough memory, and \p *from will be \c NULL;
   */
  im_err_t im_presence_get_from (im_presence_t *presence, char **from);

  /**
   *@brief get the protocol of a presence structure.
   *
   *@param[in]  presence the presence structure.
   *@param[out] prtl a pointer to the protocol.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   */
  im_err_t im_presence_get_protocol (im_presence_t *presence,
                                     im_protocol_t *prtl);

  /**
   *@brief get extended value information of a presence structure.
   *
   *@param[in]  presence the presence structure.
   *@param[in]  name the value name.
   *@param[out] value a pointer to a string to return the value.
   *                  This string should be released by developers after usage.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOSUPPORT no value respects to the name, *value will be \c NULL;
   *         - #IM_ERROR_NOMEM no memory, \p *value will be \c NULL;
   */
  im_err_t im_presence_get_ex_info (im_presence_t *presence,
                                    char *name, char **value);

  /**
   *@brief set extended value information of a presence structure.
   *
   *@param[in]  presence the presence structure.
   *@param[in]  name the value name.
   *@param[in]  value a string contain the value.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory;
   */
  im_err_t im_presence_set_ex_info (im_presence_t *presence,
                                    char *name, char *value);

  /**
   *@brief get extended value name list of a presence structure.
   *
   *@param[in]  presence the presence structure.
   *@param[out] names a pointer to a string array to return the name list.
   *                  The last element of the string array is always be \c NULL.
   *                  This string should be released by developers after usage.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOSUPPORT no value respects to the name, *value will be \c NULL;
   *         - #IM_ERROR_NOMEM no memory, \p *value will be \c NULL;
   */
  im_err_t im_presence_get_ex_name_list (im_presence_t *presence,
                                         char ***names);

  /**
   *@}
   *end of group
   */
#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif /* __LIPS_IM_PRESENCE_H*/

