/*  libim - Library of Instant Messaging and Presence enabler
 * imuri.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file imuri.h
 *@brief API definition of Instance Message Interface - im uri
 */

#ifndef __LIPS_IM_URI_H
#define __LIPS_IM_URI_H

#include "imerror.h"
#include "imtype.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  /**
   *@ingroup LiPS_IM_API
   *@defgroup IM_URI IM_URI
   *@{
   */

  /**
   *@brief enumeration for label different parts of an IM uri address.
   *Please refer to @ref IM_URI
   */
  typedef enum
  {
    IM_URI_SCHEME,              ///< scheme part, i.e. jabber, sip, sips, wv.
    IM_URI_USER,                ///< user part, i.e. mike, john.
    IM_URI_DOMAIN,              ///< domain part, i.e. google.com, wanadoo.fr.
    IM_URI_RESOURCE,            ///< resource part, i.e. pc, mobile, gaim.
    IM_URI_PARTIAL,             ///< scheme + user + domain.
    IM_URI_FULL,                ///< scheme + user + domain + resource.
    IM_URI_FULL_EXT,            ///< scheme + user + domain + resource + extensions.
    IM_URI_EXT                  ///< extension part according to protocol.
  } im_uri_part_t;

  /**
   *@struct im_uri_t
   *@brief im_uri is an opaque data structure to represent im uri address.
   */
  typedef struct _im_uri_t im_uri_t;

  /**
   *@brief create a new #im_uri_t structure.
   *
   *@param[in]  uri_str a string contains the uri address. ex. "john.smith@jabber.org/mobile".
   *@param[in]  prtl the protocol.
   *@param[out] uri a pointer to return the new #im_uri_t structure pointer
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory and @p *uri will be \c NULL;
   */
  im_err_t im_uri_new (char *uri_str, im_protocol_t prtl, im_uri_t **uri);

  /**
   *@brief free a new #im_uri_t structure.
   *
   *@param[in]  uri the #im_uri_t structure which will be freed.
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   */
  im_err_t im_uri_free (im_uri_t *uri);

  /**
   *@brief compare two uri structures with the part specified by am im_uri_part_t value.
   *
   *@param[in]  first the first #im_uri_t structure.
   *@param[in]  second the second #im_uri_t structure.
   *@param[in]  part am #im_uri_part_t value specify the comparison part
   *            availabe value:  #IM_URI_SCHEME, #IM_URI_USER, #IM_URI_DOMAIN,
   *                             #IM_URI_RESOURCE, #IM_URI_PARTIAL, #IM_URI_FULL
   *
   *@return \c TRUE, if the specified parts of two uri addresses are equivalent;
   *        else \c FALSE.
   */
  bool im_uri_cmp (im_uri_t *first, im_uri_t *second, im_uri_part_t part);

  /**
   *@brief get extended value from an #im_uri_t structure.
   *
   *@param[in]  uri the uri.
   *@param[in]  name the name of the value.
   *@param[out] value a pointer to a string which will return the value.
   *                  It should be released by programmers after usage.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOSUPPORT no value respects to the name, *value will be \c NULL;
   *         - #IM_ERROR_NOMEM no memory, @p *value will be \c NULL;
   */
  im_err_t im_uri_get_ext (im_uri_t *uri, char *name, char **value);

  /**
   *@brief set extended value for an #im_uri_t structure.
   *
   *@param[in]  uri the uri.
   *@param[in]  name the name of the value.
   *@param[in]  value the value.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory;
   *
   *@note If the name does not exist, the name-value pair will be attached to the end
   *of the uri.
   */
  im_err_t im_uri_set_ext (im_uri_t *uri, char *name, char *value);

  /**
   *@brief get extended value name list from an #im_uri_t structure.
   *
   *@param[in]  uri the uri.
   *@param[out] names a pointer to a string array which returns the value
   *                  name list. The last element of the array is always \c NULL.
   *                  It should be released by programmers after usage.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory, @p *names will be \c NULL;
   */
  im_err_t im_uri_get_ext_names (im_uri_t *uri, char ***names);

  /**
   *@brief get a string contants full-format and extension values of an #im_uri_t structure.
   *
   *The full-extension formate is used to show all the parts of the uri address.
   *@par
   *<code>ex. for a SIP uri, the full-extension string could be "sip:user@domain/resource;user=phone"</code>
   *
   *@param[in]  uri the uri
   *@param[out] str a pointer to the string which will return the full format string and extension value.
   *                It should be released by programmers after usage.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory, @p *str will be \c NULL;
   */
  im_err_t im_uri_get_full_ext (im_uri_t *uri, char **str);

  /**
   *@brief get a string of an uri address in standard format.
   *
   *The string contains scheme (for Xmpp, it's optional), user and domain parts.
   *@par
   *<code>ex. for an Xmpp uri, the string will be "user@domain"</code>
   *
   *@param[in]  uri the uri
   *@param[out] std_str a pointer to the string which will return the standard-format.
   *                    It should be released by programmers after usage.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory, @p *std_str will be \c NULL;
   */
  im_err_t im_uri_get_uri (im_uri_t *uri, char **std_str);

  /**
   *@brief get a string of an uri address in full-format.
   *
   *The string contains scheme (for Xmpp, it's optional), user, domain and resource parts.
   *@par
   *<code> ex. for an Xmpp uri, the string will be "user@domain/resource" </code>
   *
   *@param[in]  uri the uri
   *@param[out] full_str a pointer to the string which will return the full-format.
   *                     It should be released by programmers after usage.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory, @p *full_str will be \c NULL;
   */
  im_err_t im_uri_get_uri_full (im_uri_t *uri, char **full_str);

  /**
   *@brief get the protocol of an #im_uri_t structure.
   *
   *@param[in]  uri the uri
   *@param[out] prtl an #im_protocol_t pointer to return the protocol.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   */
  im_err_t im_uri_get_protocol (im_uri_t *uri, im_protocol_t *prtl);

  /**
   *@brief get the user part of an #im_uri_t structure.
   *
   *@param[in]  uri the uri
   *@param[out] user a pointer to a string to return the user part.
   *                 It should be released by programmers after usage.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory, @p *user will be \c NULL;
   */
  im_err_t im_uri_get_user (im_uri_t *uri, char **user);

  /**
   *@brief set the user part of an #im_uri_t structure.
   *
   *@param[in]  uri the uri
   *@param[in]  user a string contain the user.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory;
   */
  im_err_t im_uri_set_user (im_uri_t *uri, char *user);

  /**
   *@brief get the domain part of an #im_uri_t structure.
   *
   *@param[in]  uri the uri
   *@param[out] domain a pointer to a string to return the domain part.
   *                   It should be released by programmers after usage.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory, @p *domain will be \c NULL;
   */
  im_err_t im_uri_get_domain (im_uri_t *uri, char **domain);

  /**
   *@brief set the domain part of an #im_uri_t structure.
   *
   *@param[in]  uri the uri
   *@param[in]  domain a string contains the domain.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory;
   */
  im_err_t im_uri_set_domain (im_uri_t *uri, char *domain);

  /**
   *@brief get the resource part of an #im_uri_t structure.
   *
   *@param[in]  uri the uri
   *@param[out] res a pointer to a string to return the resource part.
   *                It should be released by programmers after usage.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory, @p *res will be \c NULL;
   */
  im_err_t im_uri_get_res (im_uri_t *uri, char **res);

  /**
   *@brief set the resource part of an #im_uri_t structure.
   *
   *@param[in]  uri the uri
   *@param[in]  res a string to contains the resource.
   *
   *@return an #im_err_t value
   *         - #IM_ERROR_NONE no error;
   *         - #IM_ERROR_NOMEM no memory;
   */
  im_err_t im_uri_set_res (im_uri_t *uri, char *res);

  /**
   *@}
   *end group.
   */
#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_IM_URI_H */

