/*  libim - Library of Instant Messaging and Presence enabler
 * imconnect.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


/**
 *@file API implememt of the Instance Message Interface - IM Connect
 *
 *@version 0.1
 */

#include "imconnect_base.h"
#include <glib.h>
#include "imtrans.h"
#include <string.h>
#include "improxy_loop.h"

im_err_t
im_conn_init_base (im_connection *conn, char *ver)
{
  conn->version = g_strdup (ver);
  conn->protocol = IM_PROTOCOL_XMPP;
  conn->status = IM_CONNECTION_DISCONNECTED;
  conn->roster = NULL;
  conn->ref = 1;
  im_rec_mutex_new (&(conn->mutex));
  return IM_ERROR_NONE;
};

im_err_t
im_conn_free_base (im_connection *conn)
{
  g_message ("xxxxx free connection: %s:%s\n", conn->server_addr,
             conn->server_port);
  im_rec_mutex_lock (conn->mutex);
  if (conn->login_uri)
    im_uri_free (conn->login_uri);
  if (conn->server_addr)
    g_free (conn->server_addr);
  if (conn->server_port)
    g_free (conn->server_port);
  if (conn->version)
    g_free (conn->version);
  //free roster
  if (conn->roster)
    im_roster_free (conn->roster);
  im_rec_mutex_unlock (conn->mutex);
  im_rec_mutex_free (conn->mutex);
  return IM_ERROR_NONE;
};

im_err_t
im_conn_login (im_connection *conn, im_sid_t sid, char *passwd,
               im_rid_t *pRid)
{
  im_err_t rt = IM_ERROR_NONE;

  im_rec_mutex_lock (conn->mutex);
  if (conn->status != IM_CONNECTION_DISCONNECTED)
  {
    if (conn->status == IM_CONNECTION_LOGINED)
      rt = IM_ERROR_LOGINED;    ///if already connected
    else
      rt = IM_ERROR_NETWORK_STATE;
    if (strcmp (conn->passwd, passwd))
      rt = IM_ERROR_PASSWORD;   ///if password is wrong
  }
  else
  {
    conn->status = IM_CONNECTION_LOGINING;
    conn->cancel_flag = FALSE;
    conn->passwd = g_strdup (passwd);
    /// initial transaction managers.
    conn->tmgr = im_trans_mgr_new ();
    conn->pendings = im_trans_mgr_new ();
    /// create a transaction
    conn->login_t =
      im_trans_new_with_sid (sid, IM_TRANS_RID_LOGIN, IM_REQ_LOGIN, NULL);
    *pRid = conn->login_t->rid;
    rt = conn->login (conn, passwd);
    if (rt != IM_ERROR_NONE)
      conn->status = IM_CONNECTION_DISCONNECTED;
  }
  im_rec_mutex_unlock (conn->mutex);
  return rt;
};

im_err_t
im_conn_login_cancel (im_connection *conn)
{
  if (conn->status == IM_CONNECTION_LOGINING)
  {
    conn->cancel_flag = TRUE;
  };
  return conn->login_cancel (conn);
};

/**logout */
///
///if pRid is NULL, no logout event will be published.
///
im_err_t
im_conn_logout (im_connection *conn, im_sid_t sid, im_rid_t *pRid)
{
  im_err_t rt = IM_ERROR_NONE;

  im_rec_mutex_lock (conn->mutex);
  if (conn->status != IM_CONNECTION_LOGINED)
  {
    rt = IM_ERROR_NETWORK_STATE;
  }
  else
  {                             // only logout if conn is already login.
    conn->status = IM_CONNECTION_LOGOUTING;
    conn->logout_t =
      im_trans_new_with_sid (sid, IM_TRANS_RID_LOGOUT, IM_REQ_LOGOUT, NULL);
    if (pRid)
    {
      *pRid = conn->logout_t->rid;
    }
    else
    {
      conn->logout_t->rid = IM_TRANS_RID_INNER_LOGOUT;
    }
    im_trans_mgr_push (conn->tmgr, conn->logout_t);
    conn->status = IM_CONNECTION_LOGOUTING;
  }
  im_rec_mutex_unlock (conn->mutex);
  return rt;
};

im_err_t
im_conn_change_presence (const im_connection *conn, const im_sid_t sid,
                         const char *uri, const im_presence_state_t pstate,
                         const char *desc, im_rid_t *pRid)
{
  im_presence_param_t *p = g_new0 (im_presence_param_t, 1);
  im_trans_t *t = NULL;

  if (!p)
    return IM_ERROR_NOMEM;
  p->uri = g_strdup (uri);
  p->desc = g_strdup (desc);
  if ((uri && !p->uri) || (desc && !p->desc))
  {
    im_presence_param_free (p);
    return IM_ERROR_NOMEM;
  }
  p->prtl = conn->protocol;
  p->pstate = pstate;
  bool logined = TRUE;

  im_rec_mutex_lock (conn->mutex);
  if (conn->status == IM_CONNECTION_LOGINED)
  {
    t = im_trans_mgr_add_with_sid (conn->tmgr, sid, IM_REQ_PRESENCE, p);
  }
  else
  {
    logined = FALSE;
  }
  im_rec_mutex_unlock (conn->mutex);
  if (!logined)
  {
    im_presence_param_free (p);
    *pRid = IM_TRANS_RID_EVT;
    return IM_ERROR_NETWORK_STATE;
  }
  if (!t)
    return IM_ERROR_NOMEM;
  *pRid = t->rid;
  return IM_ERROR_NONE;
};

im_err_t
im_conn_presence_probe (const im_connection *conn, const im_sid_t sid,
                        const char *uri, im_rid_t *pRid)
{
  char *to = g_strdup (uri);

  if (!to)
    return IM_ERROR_NOMEM;
  im_trans_t *t = NULL;
  bool logined = TRUE;

  im_rec_mutex_lock (conn->mutex);
  if (conn->status == IM_CONNECTION_LOGINED)
  {
    t = im_trans_mgr_add_with_sid (conn->tmgr, sid, IM_REQ_PRESENCE_PROBE, to);
  }
  else
  {
    logined = FALSE;
  }
  im_rec_mutex_unlock (conn->mutex);
  if (!logined)
  {
    g_free (to);
    *pRid = IM_TRANS_RID_EVT;
    return IM_ERROR_NETWORK_STATE;
  }
  if (!t)
    return IM_ERROR_NOMEM;
  *pRid = t->rid;
  return IM_ERROR_NONE;
};

im_err_t
im_conn_send_msg (const im_connection *conn, const im_sid_t sid,
                  const char *to, const void *msg, const size_t bsize,
                  const char *encode, const im_msg_type_t mtype,
                  const im_msg_ctype_t mctype, im_rid_t *pRid)
{
  im_msg_param_t *m = g_new0 (im_msg_param_t, 1);
  im_trans_t *t = NULL;

  if (!m)
    return IM_ERROR_NOMEM;
  m->uri = g_strdup (to);
  m->msg = g_memdup (msg, bsize);
  m->encode = g_strdup (encode);
  if ((to && !m->uri) || (msg && !m->msg) || (encode && !m->encode))
  {
    im_msg_param_free (m);
    return IM_ERROR_NOMEM;
  }
  m->msg_size = bsize;
  m->msg_type = mtype;
  m->msg_ctype = mctype;
  bool logined = TRUE;

  im_rec_mutex_lock (conn->mutex);
  if (conn->status == IM_CONNECTION_LOGINED)
  {
    t = im_trans_mgr_add_with_sid (conn->tmgr, sid, IM_REQ_MESSAGE, m);
  }
  else
  {
    logined = FALSE;
  }
  im_rec_mutex_unlock (conn->mutex);
  if (!logined)
  {
    im_msg_param_free (m);
    *pRid = IM_TRANS_RID_EVT;
    return IM_ERROR_NETWORK_STATE;
  }
  if (!t)
    return IM_ERROR_NOMEM;
  *pRid = t->rid;
  return IM_ERROR_NONE;
};

im_err_t
im_conn_subscribe (const im_connection *conn, const im_sid_t sid,
                   const char *to, const char *msg, const bool subscribe,
                   const char *nick, const char *group, im_rid_t *pRid)
{
  im_subs_msg_t *m = g_new0 (im_subs_msg_t, 1);
  im_trans_t *t = NULL;

  if (!m)
    return IM_ERROR_NOMEM;
  m->contactor = g_strdup (to);
  if (!m->contactor)
    return IM_ERROR_NOMEM;
  m->msg = g_strdup (msg);
  if (!m->msg)
    return IM_ERROR_NOMEM;
  m->nick = g_strdup (nick);
  if (!m->nick)
    return IM_ERROR_NOMEM;
  m->group = g_strdup (group);
  if (!m->group)
    return IM_ERROR_NOMEM;
  m->subscribe = subscribe;
  bool logined = TRUE;

  im_rec_mutex_lock (conn->mutex);
  if (conn->status == IM_CONNECTION_LOGINED)
  {
    t = im_trans_mgr_add_with_sid (conn->tmgr, sid, IM_REQ_SUBSCRIBE, m);
  }
  else
  {
    logined = FALSE;
  }
  im_rec_mutex_unlock (conn->mutex);
  if (!logined)
  {
    im_subs_msg_free (m);
    *pRid = IM_TRANS_RID_EVT;
    return IM_ERROR_NETWORK_STATE;
  }
  if (!t)
    return IM_ERROR_NOMEM;
  *pRid = t->rid;
  return IM_ERROR_NONE;
};

im_err_t
im_conn_accept (const im_connection *conn, const im_sid_t sid,
                const char *to, const char *msg, const bool accept,
                im_rid_t *pRid)
{
  im_accept_msg_t *m = g_new0 (im_accept_msg_t, 1);
  im_trans_t *t = NULL;

  if (!m)
    return IM_ERROR_NOMEM;
  m->contactor = g_strdup (to);
  if (!m->contactor)
    return IM_ERROR_NOMEM;
  m->msg = g_strdup (msg);
  if (!m->msg)
    return IM_ERROR_NOMEM;
  m->accept = accept;
  bool logined = TRUE;

  im_rec_mutex_lock (conn->mutex);
  if (conn->status == IM_CONNECTION_LOGINED)
  {
    t = im_trans_mgr_add_with_sid (conn->tmgr, sid, IM_REQ_ACCEPTANCE, m);
  }
  else
  {
    logined = FALSE;
  }
  im_rec_mutex_unlock (conn->mutex);
  if (!logined)
  {
    im_accept_msg_free (m);
    *pRid = IM_TRANS_RID_EVT;
    return IM_ERROR_NETWORK_STATE;
  }
  if (!t)
    return IM_ERROR_NOMEM;
  *pRid = t->rid;
  return IM_ERROR_NONE;
};

im_err_t
im_conn_get_rosters (const im_connection *conn, const im_sid_t sid,
                     im_rid_t *pRid)
{
  im_trans_t *t = NULL;
  bool logined = TRUE;

  im_rec_mutex_lock (conn->mutex);
  if (conn->status == IM_CONNECTION_LOGINED)
  {
    t = im_trans_mgr_add_with_sid (conn->tmgr, sid, IM_REQ_ROSTER, NULL);
  }
  else
  {
    logined = FALSE;
  }
  im_rec_mutex_unlock (conn->mutex);
  if (!logined)
  {
    *pRid = IM_TRANS_RID_EVT;
    return IM_ERROR_NETWORK_STATE;
  }
  if (!t)
    return IM_ERROR_NOMEM;
  *pRid = t->rid;
  return IM_ERROR_NONE;
};

im_err_t
im_conn_buddy_op (const im_connection *conn, const im_sid_t sid,
                  const char *contactor, const char *nickname,
                  const char *group, const im_buddy_operation_t op_type,
                  im_rid_t *pRid)
{
  im_roster_item_t *buddy = (im_roster_item_t *) g_new0 (im_roster_item_t, 1);

  if (!buddy)
    return IM_ERROR_NOMEM;
  buddy->contactor = g_strdup (contactor);
  buddy->nickname = g_strdup (nickname);
  buddy->group = g_strdup (group);
  if ((contactor && !buddy->contactor) ||
      (nickname && !buddy->nickname) || (group && !buddy->nickname))
    return IM_ERROR_NOMEM;
  im_buddy_op_t *op = NULL;

  if (im_buddy_op_new (buddy, op_type, &op) != IM_ERROR_NONE)
    return IM_ERROR_NOMEM;
  im_trans_t *t = NULL;
  bool logined = TRUE;

  im_rec_mutex_lock (conn->mutex);
  if (conn->status == IM_CONNECTION_LOGINED)
  {
    t = im_trans_mgr_add_with_sid (conn->tmgr, sid, IM_REQ_BUDDY, op);
  }
  else
  {
    logined = FALSE;
  }
  im_rec_mutex_unlock (conn->mutex);
  if (!logined)
  {
    *pRid = IM_TRANS_RID_EVT;
    return IM_ERROR_NETWORK_STATE;
  }
  if (!t)
    return IM_ERROR_NOMEM;
  *pRid = t->rid;
  return IM_ERROR_NONE;
};

im_err_t
im_conn_buddies_synch (const im_connection *conn, const im_sid_t sid,
                       im_rid_t *pRid)
{
  im_trans_t *t = NULL;
  bool logined = TRUE;

  im_rec_mutex_lock (conn->mutex);
  if (conn->status == IM_CONNECTION_LOGINED)
  {
    t = im_trans_mgr_add_with_sid (conn->tmgr, sid, IM_REQ_SYNCH, NULL);
  }
  else
  {
    logined = FALSE;
  }
  im_rec_mutex_unlock (conn->mutex);
  if (!logined)
  {
    *pRid = IM_TRANS_RID_EVT;
    return IM_ERROR_NETWORK_STATE;
  }
  if (!t)
    return IM_ERROR_NOMEM;
  *pRid = t->rid;
  return IM_ERROR_NONE;
};

im_err_t
im_conn_on_login (im_connection *conn, im_err_code_t err_code)
{
  im_rec_mutex_lock (conn->mutex);
  im_trans_t *t = conn->login_t;

  conn->login_t = NULL;
  if (err_code == IM_ERR_CODE_NONE)
  {
    t->status = IM_TRANS_OK;
  }
  else
  {
    t->status = IM_TRANS_FAIL;
  }
  t->err_code = err_code;
  im_proxy_trans_notify (t);
  if (err_code == IM_ERR_CODE_NONE)
  {
    conn->status = IM_CONNECTION_LOGINED;
  }
  else
  {
    conn->status = IM_CONNECTION_DISCONNECTED;
    if (conn->tmgr)
      im_trans_mgr_free (conn->tmgr);
    if (conn->pendings)
      im_trans_mgr_free (conn->pendings);
  }
  im_rec_mutex_unlock (conn->mutex);
  return IM_ERROR_NONE;
};

im_err_t
im_conn_on_logout (im_connection *conn)
{
  im_rec_mutex_lock (conn->mutex);
  im_trans_t *t = conn->logout_t;

  t->status = IM_TRANS_OK;
  conn->logout_t = NULL;
  if (t->rid == IM_TRANS_RID_LOGOUT)
  {                             /// if this function is caused by logout request
    im_proxy_trans_notify (t);
  }
  else if (t->rid == IM_TRANS_RID_INNER_LOGOUT)
  {
    /// else it is caused by close session request.
    im_trans_free (t);
  }
  if (conn->tmgr)
    im_trans_mgr_free (conn->tmgr);
  if (conn->pendings)
    im_trans_mgr_free (conn->pendings);
  conn->status = IM_CONNECTION_DISCONNECTED;
  im_rec_mutex_unlock (conn->mutex);
  return IM_ERROR_NONE;
};

im_err_t
im_conn_on_presence (const im_connection *conn, const char *uri,
                     const im_presence_state_t pstate,
                     const im_protocol_t prtl, const char *desc)
{
  im_presence_param_t *p = g_new0 (im_presence_param_t, 1);

  if (!p)
    return IM_ERROR_NOMEM;
  p->uri = g_strdup (uri);
  if (uri && !p->uri)
    return IM_ERROR_NOMEM;
  p->desc = g_strdup (desc);
  if (desc && !p->desc)
    return IM_ERROR_NOMEM;
  p->prtl = prtl;
  p->pstate = pstate;
  im_trans_t *t = im_trans_new (NULL, IM_TRANS_RID_EVT, IM_EVT_PRESENCE, NULL);

  if (!t)
    return IM_ERROR_NOMEM;
  t->out = p;
  t->status = IM_TRANS_OK;
  t->conn = conn;
  im_proxy_trans_notify (t);
  return IM_ERROR_NONE;
};

im_err_t
im_conn_on_message (const im_connection *conn, im_msg_param_t *msg)
{
  if (!msg)
    return IM_ERROR_NULL;
  im_trans_t *t = im_trans_new (NULL, IM_TRANS_RID_EVT, IM_EVT_MESSAGE, NULL);

  if (!t)
    return IM_ERROR_NOMEM;
  t->out = msg;
  t->status = IM_TRANS_OK;
  t->conn = conn;
  im_proxy_trans_notify (t);
  return IM_ERROR_NONE;
};

im_err_t
im_conn_on_subscribe (const im_connection *conn, const char *from,
                      const char *msg, const bool subscribe)
{
  im_subs_msg_t *m = g_new0 (im_subs_msg_t, 1);

  if (!m)
    return IM_ERROR_NOMEM;
  m->contactor = g_strdup (from);
  if (from && !m->contactor)
    return IM_ERROR_NOMEM;
  m->msg = g_strdup (msg);
  if (msg && !m->msg)
    return IM_ERROR_NOMEM;
  m->subscribe = subscribe;
  im_trans_t *t = im_trans_new (NULL, IM_TRANS_RID_EVT, IM_EVT_SUBSCRIBE, NULL);

  if (!t)
    return IM_ERROR_NOMEM;
  t->out = m;
  t->status = IM_TRANS_OK;
  t->conn = conn;
  im_proxy_trans_notify (t);
  return IM_ERROR_NONE;
};

im_err_t
im_conn_on_accept (const im_connection *conn, const char *from,
                   const char *msg, const bool accept)
{
  im_accept_msg_t *m = g_new0 (im_accept_msg_t, 1);

  if (!m)
    return IM_ERROR_NOMEM;
  m->contactor = g_strdup (from);
  if (from && !m->contactor)
    return IM_ERROR_NOMEM;
  m->msg = g_strdup (msg);
  if (msg && !m->msg)
    return IM_ERROR_NOMEM;
  m->accept = accept;
  im_trans_t *t =
    im_trans_new (NULL, IM_TRANS_RID_EVT, IM_EVT_ACCEPTANCE, NULL);
  if (!t)
    return IM_ERROR_NOMEM;
  t->out = m;
  t->status = IM_TRANS_OK;
  t->conn = conn;
  im_proxy_trans_notify (t);
  return IM_ERROR_NONE;
};

im_err_t
im_conn_on_error (im_connection *conn, const im_err_code_t err_code,
                  const char *msg)
{
  im_rec_mutex_lock (conn->mutex);
  conn->status = IM_CONNECTION_DISCONNECTED;
  if (conn->tmgr)
    im_trans_mgr_free (conn->tmgr);
  if (conn->pendings)
    im_trans_mgr_free (conn->pendings);
  im_rec_mutex_unlock (conn->mutex);

  im_network_error_t *m = g_new0 (im_network_error_t, 1);

  if (!m)
    return IM_ERROR_NOMEM;
  m->msg = g_strdup (msg);
  if (msg && !m->msg)
    return IM_ERROR_NOMEM;
  m->err_type = err_code;
  im_trans_t *t = im_trans_new (NULL, IM_TRANS_RID_EVT, IM_EVT_ERROR, NULL);

  if (!t)
    return IM_ERROR_NOMEM;
  t->out = m;
  t->status = IM_TRANS_OK;
  t->conn = conn;
  im_proxy_trans_notify (t);
  return IM_ERROR_NONE;
};

im_err_t
im_conn_on_roster (const im_connection *conn)
{
  if (!conn->roster)
  {
    g_warning ("roster is missed\n");
    return IM_ERROR_GENERAL;
  }
  im_trans_t *t = im_trans_new (NULL, IM_TRANS_RID_EVT, IM_EVT_ROSTER, NULL);

  if (!t)
    return IM_ERROR_NOMEM;
  t->out = conn->roster;
  t->status = IM_TRANS_OK;
  t->conn = conn;
  im_proxy_trans_notify (t);
  return IM_ERROR_NONE;
};

im_err_t
im_conn_on_buddy_op (const im_connection *conn, im_buddy_op_t *op)
{
  im_trans_t *t = im_trans_new (NULL, IM_TRANS_RID_EVT, IM_EVT_BUDDY, NULL);

  if (!t)
    return IM_ERROR_NOMEM;
  t->out = op;
  t->status = IM_TRANS_OK;
  t->conn = conn;
  im_proxy_trans_notify (t);
  return IM_ERROR_NONE;
};

im_connection *
im_conn_ref (im_connection *conn)
{
  g_assert (conn);
  g_message ("in_conn_ref ++\n");
  g_atomic_int_inc ((gint *) & (conn->ref));
  return conn;
};

im_connection *
im_conn_unref (im_connection *conn)
{
  g_assert (conn);
  g_message ("in_conn_ref --\n");
  if (g_atomic_int_dec_and_test ((gint *) & (conn->ref)))
  {
    im_rec_mutex_lock (conn->mutex);
    if (conn->status == IM_CONNECTION_LOGINED)
    {
      im_conn_logout (conn, 0, NULL);   //this invocation will not generate logout event.
    }
    im_rec_mutex_unlock (conn->mutex);
    conn->free (conn);
    conn = NULL;
  }
  return conn;
};

im_trans_mgr_t *
im_conn_get_trans_mgr (im_connection *conn)
{
  return conn->tmgr;
};

im_trans_mgr_t *
im_conn_get_pendings (im_connection *conn)
{
  return conn->pendings;
};

