/*  libim - Library of Instant Messaging and Presence enabler
 * imtrans.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


/**
 *@file API definition of the Instance Message Interface - IM Transaction
 *
 *@version 0.1
 */

#include "imtrans.h"
#include <glib.h>
#include "string.h"
#include "imconnect.h"
#include "improxy.h"

struct _im_trans_mgr_t
{
  //im_hash*hs;
  GQueue *queue;
  GMutex *mutex;
  im_rid_t rid_seed;
};

void
im_presence_param_free (im_presence_param_t *p)
{
  g_assert (p);
  g_free (p->uri);
  g_free (p->desc);
  g_free (p);
};

void
im_msg_param_free (im_msg_param_t *msg)
{
  g_assert (msg);
  g_free (msg->uri);
  g_free (msg->encode);
  g_free (msg->msg);
  g_free (msg);
};

bool
im_trans_equalto (const im_trans_t *me, const im_trans_t *other)
{
  return strcmp ((char *) (me->tid), (char *) (other->tid)) == 0;
};

im_trans_t *
im_trans_new (im_session_t *session, im_rid_t rid,
              im_trans_type_t type, void *in_data)
{
  //g_assert(session);
  im_trans_t *t = g_new0 (im_trans_t, 1);

  g_assert (t);
  t->session = session;
  t->rid = rid;
  t->status = IM_TRANS_INIT;
  t->in = in_data;
  t->type = type;
  t->err_code = IM_ERR_CODE_NONE;
  return t;
};

im_trans_t *
im_trans_new_with_sid (im_sid_t sid, im_rid_t rid,
                       im_trans_type_t type, void *in_data)
{
  im_session_t *session = NULL;

  if (sid != 0)
  {
    session = im_proxy_get_session (im_proxy_get (), sid);
    if (!session)
      return NULL;
  }
  im_trans_t *t = g_new0 (im_trans_t, 1);

  g_assert (t);
  t->session = session;
  t->rid = rid;
  t->status = IM_TRANS_INIT;
  t->in = in_data;
  t->type = type;
  return t;
};

void
im_trans_free (im_trans_t *trans)
{
  g_assert (trans);
  switch (trans->type)
  {
    case IM_REQ_LOGIN:
      if (trans->in)
        g_free ((char *) (trans->in));
      break;
    case IM_REQ_PRESENCE:
    case IM_EVT_PRESENCE:
      if (trans->in)
        im_presence_param_free ((im_presence_param_t *) trans->in);
      if (trans->out)
        im_presence_param_free ((im_presence_param_t *) trans->out);
      break;
    case IM_REQ_PRESENCE_PROBE:
      if (trans->in)
        g_free (trans->in);
      if (trans->out)
        im_presence_param_free ((im_presence_param_t *) trans->out);
      break;
    case IM_REQ_MESSAGE:
    case IM_EVT_MESSAGE:
      if (trans->in)
        im_msg_param_free ((im_msg_param_t *) trans->in);
      if (trans->out)
        im_msg_param_free ((im_msg_param_t *) trans->out);
      break;
    case IM_REQ_SUBSCRIBE:
    case IM_EVT_SUBSCRIBE:
      if (trans->in)
        im_subs_msg_free ((im_subs_msg_t *) (trans->in));
      if (trans->out)
        im_subs_msg_free ((im_subs_msg_t *) (trans->out));
      break;
    case IM_REQ_ACCEPTANCE:
    case IM_EVT_ACCEPTANCE:
      if (trans->in)
        im_accept_msg_free ((im_accept_msg_t *) (trans->in));
      if (trans->out)
        im_accept_msg_free ((im_accept_msg_t *) (trans->out));
      break;
    case IM_REQ_ROSTER:
    case IM_EVT_ROSTER:
      //-----roster will be destoried by connection.
      //    if(trans->in)
      //      im_roster_free((im_roster_t *)(trans->in));
      //    if(trans->out)
      //      im_roster_free((im_roster_t *)(trans->out));
      break;
    case IM_REQ_BUDDY:
      if (trans->in)
        im_buddy_op_free_all ((im_buddy_op_t *) (trans->in));
      break;
    case IM_EVT_BUDDY:
      if (trans->out)
        im_buddy_op_free_all ((im_buddy_op_t *) (trans->out));
      break;
    default:
      break;
  }
  if (trans->tid)
    g_free ((char *) (trans->tid));
  g_free (trans);
};

im_err_t
im_trans_cancel (im_trans_t *trans)
{
  return IM_ERROR_NOSUPPORT;
};

req_state_t
im_trans_req_state (im_trans_t *trans)
{
  req_state_t status = EVT_STATE_NOTIFY;

  switch (trans->status)
  {
    case IM_TRANS_OK:
    case IM_TRANS_FAIL:
      status = EVT_STATE_COMPLIETED;
      break;
    case IM_TRANS_CANCEL:
    case IM_TRANS_TIMEOUT:
    case IM_TRANS_NOSUPPORT:
      status = EVT_STATE_CANCELED;
      break;
    default:
      break;
  }
  return status;
};

/* 
static gboolean im_trans_find(gpointer key, gpointer value, gpointer user_data)
{
  return strcmp((char*)(((im_trans_t*)value)->tid), (char*)user_data) == 0;
};*/

im_trans_mgr_t *
im_trans_mgr_new ()
{
  im_trans_mgr_t *tm = g_new0 (im_trans_mgr_t, 1);

  g_assert (tm);
  //tm->hs = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, im_trans_free);
  tm->queue = g_queue_new ();
  g_assert (tm->queue);
  tm->mutex = g_mutex_new ();
  g_assert (tm->mutex);
  tm->rid_seed = IM_TRANS_RID_BEGIN;
  return tm;
};

void
im_trans_mgr_free (im_trans_mgr_t *t_mgr)
{
  g_mutex_lock (t_mgr->mutex);
  //g_hash_table_destroy((GHashTable*)(t_mgr->hs));
  g_queue_free (t_mgr->queue);
  g_mutex_unlock (t_mgr->mutex);
  g_mutex_free (t_mgr->mutex);
  g_free (t_mgr);
};

im_trans_t *
im_trans_mgr_add (im_trans_mgr_t *t_mgr, im_session_t *session,
                  im_trans_type_t type, void *in_data)
{
  g_assert (t_mgr);
  g_mutex_lock (t_mgr->mutex);
  im_trans_t *t = im_trans_new (session, ++(t_mgr->rid_seed), type, in_data);

  g_assert (t);
  //g_hash_table_insert((GHashTable*)(t_mgr->hs), t->rid, t);
  g_queue_push_head (t_mgr->queue, (gpointer) t);
  g_mutex_unlock (t_mgr->mutex);
  return t;
};

im_trans_t *
im_trans_mgr_add_with_sid (im_trans_mgr_t *t_mgr, im_sid_t sid,
                           im_trans_type_t type, void *in_data)
{
  g_assert (t_mgr);
  g_mutex_lock (t_mgr->mutex);
  im_trans_t *t =
    im_trans_new_with_sid (sid, ++(t_mgr->rid_seed), type, in_data);
  if (t)
    g_queue_push_head (t_mgr->queue, (gpointer) t);
  g_mutex_unlock (t_mgr->mutex);
  return t;
};

static gint
im_trans_find_by_tid (im_trans_t *t, im_tid_t tid)
{
  return strcmp ((char *) (t->tid), (char *) tid);
};

static gint
im_trans_find_by_rid (im_trans_t *t, im_rid_t rid)
{
  return (t->rid - rid);
};

im_trans_t *
im_trans_mgr_get_by_tid (im_trans_mgr_t *t_mgr, im_tid_t tid)
{
  g_assert (t_mgr);
  im_trans_t *t = NULL;

  g_mutex_lock (t_mgr->mutex);
  //im_trans_t*t = (im_trans_t*)g_hash_table_find(
  //        (GHashTable*)(t_mgr->hs), im_trans_find, tid);
  GList *l = g_queue_find_custom (t_mgr->queue,
                                  (gconstpointer) tid,
                                  (GCompareFunc) im_trans_find_by_tid);
  if (l)
    t = (im_trans_t *) (l->data);
  g_mutex_unlock (t_mgr->mutex);
  return t;
};

im_trans_t *
im_trans_mgr_get_by_rid (im_trans_mgr_t *t_mgr, im_rid_t rid)
{
  g_assert (t_mgr);
  im_trans_t *t = NULL;

  g_mutex_lock (t_mgr->mutex);
  //im_trans_t*t = (im_trans_t*)g_hash_table_lookup((GHashTable*)(t_mgr->hs), rid);
  GList *l = g_queue_find_custom (t_mgr->queue,
                                  GUINT_TO_POINTER (rid),
                                  (GCompareFunc) im_trans_find_by_rid);
  if (l)
    t = (im_trans_t *) (l->data);
  g_mutex_unlock (t_mgr->mutex);
  return t;
};

im_trans_t *
im_trans_mgr_remove_by_tid (im_trans_mgr_t *t_mgr, im_tid_t tid)
{
  g_assert (t_mgr);
  im_trans_t *rt = NULL;

  g_mutex_lock (t_mgr->mutex);
  rt = im_trans_mgr_get_by_tid (t_mgr, tid);
  if (rt)
    g_queue_remove (t_mgr->queue, (gconstpointer) rt);
  g_mutex_unlock (t_mgr->mutex);
  return rt;
};

im_trans_t *
im_trans_mgr_remove_by_rid (im_trans_mgr_t *t_mgr, im_rid_t rid)
{
  g_assert (t_mgr);
  im_trans_t *rt = NULL;

  g_mutex_lock (t_mgr->mutex);
  rt = im_trans_mgr_get_by_rid (t_mgr, rid);
  if (rt)
    g_queue_remove (t_mgr->queue, (gconstpointer) rt);
  g_mutex_unlock (t_mgr->mutex);
  return rt;
};

bool
im_trans_mgr_has_tid (im_trans_mgr_t *t_mgr, im_tid_t tid)
{
  g_assert (t_mgr);
  g_mutex_lock (t_mgr->mutex);
  bool r = (im_trans_mgr_get_by_tid (t_mgr, tid) != NULL);

  g_mutex_unlock (t_mgr->mutex);
  return r;
};

bool
im_trans_mgr_has_rid (im_trans_mgr_t *t_mgr, im_rid_t rid)
{
  g_assert (t_mgr);
  g_mutex_lock (t_mgr->mutex);
  bool r = (im_trans_mgr_get_by_rid (t_mgr, rid) != NULL);

  g_mutex_unlock (t_mgr->mutex);
  return r;
};

uint32_t
im_trans_mgr_len (im_trans_mgr_t *t_mgr)
{
  g_assert (t_mgr);
  g_mutex_lock (t_mgr->mutex);
  uint32_t r = g_queue_get_length (t_mgr->queue);

  g_mutex_unlock (t_mgr->mutex);
  return r;
};

im_trans_t *
im_trans_mgr_push (im_trans_mgr_t *t_mgr, im_trans_t *t)
{
  g_assert (t_mgr);
  g_mutex_lock (t_mgr->mutex);
  g_queue_push_head (t_mgr->queue, (gpointer) t);
  g_mutex_unlock (t_mgr->mutex);
  return t;
};

im_trans_t *
im_trans_mgr_try_pop (im_trans_mgr_t *t_mgr)
{
  g_assert (t_mgr);
  g_mutex_lock (t_mgr->mutex);
  im_trans_t *t = (im_trans_t *) g_queue_pop_tail (t_mgr->queue);

  g_mutex_unlock (t_mgr->mutex);
  return t;
};

